#pragma once

#include "ATRInterface.h"
#include "LanFile.h"
#include "resource.h"       // main symbols


#if defined(_WIN32_WCE) && !defined(_CE_DCOM) && !defined(_CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA)
#error "Single-threaded COM objects are not properly supported on Windows CE platform, such as the Windows Mobile platforms that do not include full DCOM support. Define _CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA to force ATL to support creating single-thread COM object's and allow use of it's single-threaded COM object implementations. The threading model in your rgs file was set to 'Free' as that is the only threading model supported in non DCOM Windows CE platforms."
#endif


/***************************************************************************************************/
// Class IATRLanguageImpl
/***************************************************************************************************/
///
/// \ingroup ATRInterface
///
/// \brief Implements IATRLanguage methods
///
///   Implementations must be in a separate class in order to inherit implementations to the
///   interfaces which inherits IATRLanguage interface.
///
/// \author tsalonen
///
/// \date 07.06.2006
///
/// \sa CATRLanguage
/***************************************************************************************************/

template <class T>
class IATRLanguageImpl : public T
{
// implements IATRLanguage methods
public:

  // NOTE:
  // Implementation has to be in .h file because compiler has to know full
  // specification of template functions in order to use them elsewhere


  /***************************************************************************************************/
  //* get_Language
  /***************************************************************************************************/
  ///
  /// \brief Call this to get ID of current language.
  ///
  /// \param [in] pVal:  Desired window style specified in ctDlgBoxStyle_e.
  ///
  /// \return S_OK if successful
  ///
  /// \author tsalonen
  ///
  /// \date 09.06.2006
  ///
  /// \sa put_Language(LONG newVal)
  /***************************************************************************************************/

  STDMETHODIMP get_Language(LONG* pVal)
  {
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    *pVal = theApp.GetLanguage();

    return S_OK;
  }


  /***************************************************************************************************/
  //* put_Language
  /***************************************************************************************************/
  ///
  /// \brief Changes current language.
  ///
  ///   Note that language is changed for all interfaces created from this object.
  ///
  /// \param [in] newVal:  Desired window style specified in ctDlgBoxStyle_e.
  ///
  /// \return S_OK if successful
  ///
  /// \author tsalonen
  ///
  /// \date 09.06.2006
  ///
  /// \sa get_Language(LONG* pVal)
  /***************************************************************************************************/

  STDMETHODIMP put_Language(LONG newVal)
  {
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    theApp.SetLanguage( newVal );

    return S_OK;
  }


  /***************************************************************************************************/
  //* put_LanguageResource
  /***************************************************************************************************/
  ///
  /// \brief Call this to set desired language resource ID.
  ///
  ///   This must be called in order to specify which language resource interface is using in the
  ///   LANGUAGE section of resource dll.
  ///
  /// \param [in] newVal:  Desired window style specified in ctDlgBoxStyle_e.
  ///
  /// \return S_OK if successful
  ///
  /// \author tsalonen
  ///
  /// \date 09.06.2006
  /***************************************************************************************************/

  STDMETHODIMP put_LanguageResource(LONG newVal)
  {
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    miLangResource = (UINT)newVal;

    return S_OK;
  }

protected:


  /***************************************************************************************************/
  //* GetTranslation
  /***************************************************************************************************/
  ///
  /// \brief Returns specific translation.
  ///
  ///   Call this to return specific translation from currently loaded language resource. If current
  ///   language is changed then specific resource section is reloaded before returning the
  ///   translation.
  ///
  /// \param [in] roGroup:  Desired window style specified in ctDlgBoxStyle_e.
  ///
  /// \param [in] lKey:  Desired window style specified in ctDlgBoxStyle_e.
  ///
  /// \param [in] roDefault:  Desired window style specified in ctDlgBoxStyle_e.
  ///
  /// \return Requested trasnlation if successful and translation is found; otherwise specifed default
  ///         value.
  ///
  /// \author tsalonen
  ///
  /// \date 09.06.2006
  /***************************************************************************************************/

  CString GetTranslation( const CString& roGroup, const long lKey, const CString& roDefault = _T("") )
  {
    EHSTART

    if( moLanFile.GetResourceDll() != theApp.GetResourceDll() )
    {
      theApp.LoadLangFile( moLanFile, miLangResource );
    }

    return( moLanFile.GetString( roGroup, lKey, roDefault ) );

    EHEND_RET( roDefault )
  }


private:

  CLanFile    moLanFile;        ///< Object containing all translations
  UINT        miLangResource;   ///< Resource ID of LAN file to get translations
};



/***************************************************************************************************/
// Class CATRLanguage
/***************************************************************************************************/
/// 
/// \ingroup ATRInterface
///
/// \brief Class object for IATRLanguage interface.
///
///   Implementation must be in the separate class in order to inherit implementations to the
///   co-classes that implements interfaces derived from IATRLanguage interface.
///
/// \author tsalonen
///
/// \date 07.06.2006
///
/// \sa IATRLanguageImpl
/***************************************************************************************************/

class ATL_NO_VTABLE CATRLanguage :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CATRLanguage, &CLSID_ATRLanguage>,
  public IATRLanguageImpl<IDispatchImpl<IATRLanguage, &IID_IATRLanguage, &LIBID_ATRInterfaceLib>>
{
public:
	CATRLanguage()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_ATRLANGUAGE)


BEGIN_COM_MAP(CATRLanguage)
	COM_INTERFACE_ENTRY(IATRLanguage)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

	DECLARE_PROTECT_FINAL_CONSTRUCT()


	/***************************************************************************************************/
	//* FinalConstruct
	/***************************************************************************************************/
	///
	/// \brief Called during the final stages of object construction
	///
	///   This method performs any final initialization on the m_contained member.
	///
	/// \return A standard HRESULT value.
	///
	/// \author tsalonen
	///
	/// \date 09.06.2006
	/***************************************************************************************************/

	HRESULT FinalConstruct()
	{
		return S_OK;
	}


	/***************************************************************************************************/
	//* FinalRelease
	/***************************************************************************************************/
	///
	/// \brief Called during object destruction
	///
	///   This method frees the m_contained member.
	///
	/// \return A standard HRESULT value.
	///
	/// \author tsalonen
	///
	/// \date 09.06.2006
	/***************************************************************************************************/

	void FinalRelease()
	{
	}
};

OBJECT_ENTRY_AUTO(__uuidof(ATRLanguage), CATRLanguage)













